@@ -149,12 +149,6 @@ execTuplesHashPrepare(int numCols,
  *
  * Note that keyColIdx, eqfunctions, and hashfunctions must be allocated in
  * storage that will live as long as the hashtable does.
- *
- * LookupTupleHashEntry, FindTupleHashEntry, and related functions may leak
- * memory in the tempcxt.  It is caller's responsibility to reset that context
- * reasonably often, typically once per tuple.  (We do it that way, rather
- * than managing an extra context within the hashtable, because in many cases
- * the caller can specify a tempcxt that it needs to reset per-tuple anyway.)
  */
 TupleHashTable
 BuildTupleHashTableExt(PlanState *parent,
@@ -231,8 +225,9 @@ BuildTupleHashTableExt(PlanState *parent,
 	allow_jit = metacxt != tablecxt;
 
 	/* build comparator for all columns */
+	/* XXX: should we support non-minimal tuples for the inputslot? */
 	hashtable->tab_eq_func = ExecBuildGroupingEqual(inputDesc, inputDesc,
-													NULL, &TTSOpsMinimalTuple,
+													&TTSOpsMinimalTuple, &TTSOpsMinimalTuple,
 													numCols,
 													keyColIdx, eqfuncoids, collations,
 													allow_jit ? parent : NULL);
