@@ -2455,12 +2455,12 @@ log_line_prefix(StringInfo buf, ErrorData *edata)
 				{
 					char		strfbuf[128];
 
-					snprintf(strfbuf, sizeof(strfbuf) - 1, "%" INT64_MODIFIER "x.%x",
-							 MyStartTime, MyProcPid);
+					snprintf(strfbuf, sizeof(strfbuf) - 1, "%lx.%x",
+							 (long) (MyStartTime), MyProcPid);
 					appendStringInfo(buf, "%*s", padding, strfbuf);
 				}
 				else
-					appendStringInfo(buf, "%" INT64_MODIFIER "x.%x", MyStartTime, MyProcPid);
+					appendStringInfo(buf, "%lx.%x", (long) (MyStartTime), MyProcPid);
 				break;
 			case 'p':
 				if (padding != 0)
@@ -2734,7 +2734,7 @@ write_csvlog(ErrorData *edata)
 	appendStringInfoChar(&buf, ',');
 
 	/* session id */
-	appendStringInfo(&buf, "%" INT64_MODIFIER "x.%x", MyStartTime, MyProcPid);
+	appendStringInfo(&buf, "%lx.%x", (long) MyStartTime, MyProcPid);
 	appendStringInfoChar(&buf, ',');
 
 	/* Line number */
@@ -3444,24 +3444,13 @@ write_stderr(const char *fmt,...)
 {
 	va_list		ap;
 
-	va_start(ap, fmt);
-	vwrite_stderr(fmt, ap);
-	va_end(ap);
-}
-
-
-/*
- * Write errors to stderr (or by equal means when stderr is
- * not available) - va_list version
- */
-void
-vwrite_stderr(const char *fmt, va_list ap)
-{
 #ifdef WIN32
 	char		errbuf[2048];	/* Arbitrary size? */
 #endif
 
 	fmt = _(fmt);
+
+	va_start(ap, fmt);
 #ifndef WIN32
 	/* On Unix, we just fprintf to stderr */
 	vfprintf(stderr, fmt, ap);
@@ -3484,6 +3473,7 @@ vwrite_stderr(const char *fmt, va_list ap)
 		fflush(stderr);
 	}
 #endif
+	va_end(ap);
 }
 
 
