@@ -3,7 +3,7 @@
  * random.c
  *	  random() wrapper
  *
- * Portions Copyright (c) 1996-2019, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2020, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
@@ -19,7 +19,7 @@
 
 
 long
-random()
+random(void)
 {
 	return pg_lrand48();
 }
