@@ -8,26 +8,32 @@ CREATE TABLE PATH_TBL (f1 path);
 
 INSERT INTO PATH_TBL VALUES ('[(1,2),(3,4)]');
 
-INSERT INTO PATH_TBL VALUES ('((1,2),(3,4))');
+INSERT INTO PATH_TBL VALUES (' ( ( 1 , 2 ) , ( 3 , 4 ) ) ');
 
-INSERT INTO PATH_TBL VALUES ('[(0,0),(3,0),(4,5),(1,6)]');
+INSERT INTO PATH_TBL VALUES ('[ (0,0),(3,0),(4,5),(1,6) ]');
 
-INSERT INTO PATH_TBL VALUES ('((1,2),(3,4))');
+INSERT INTO PATH_TBL VALUES ('((1,2) ,(3,4 ))');
 
-INSERT INTO PATH_TBL VALUES ('1,2 ,3,4');
+INSERT INTO PATH_TBL VALUES ('1,2 ,3,4 ');
 
-INSERT INTO PATH_TBL VALUES ('[1,2,3, 4]');
+INSERT INTO PATH_TBL VALUES (' [1,2,3, 4] ');
 
-INSERT INTO PATH_TBL VALUES ('[11,12,13,14]');
+INSERT INTO PATH_TBL VALUES ('((10,20))');	-- Only one point
 
-INSERT INTO PATH_TBL VALUES ('(11,12,13,14)');
+INSERT INTO PATH_TBL VALUES ('[ 11,12,13,14 ]');
+
+INSERT INTO PATH_TBL VALUES ('( 11,12,13,14) ');
 
 -- bad values for parser testing
+INSERT INTO PATH_TBL VALUES ('[]');
+
 INSERT INTO PATH_TBL VALUES ('[(,2),(3,4)]');
 
 INSERT INTO PATH_TBL VALUES ('[(1,2),(3,4)');
 
-SELECT f1 FROM PATH_TBL;
+INSERT INTO PATH_TBL VALUES ('(1,2,3,4');
+
+INSERT INTO PATH_TBL VALUES ('(1,2),(3,4)]');
 
 SELECT '' AS count, f1 AS open_path FROM PATH_TBL WHERE isopen(f1);
 
